package org.glassfish.devtest.pooling.monitor.client.utils;

import java.util.HashMap;
import java.util.Map;

public class JDBCMonitorData implements Cloneable{

    public Map<String, Integer> numconnused = new HashMap<String, Integer>(3);
    public Map<String, Integer> numconnfree = new HashMap<String, Integer>(3);
    public int numconncreated = 0;
    public int numconndestroyed = 0;
    public int numconnacqired = 0;
    public int numconnreleased = 0;
    public int numconnfailedvalidation = 0;
    public int numconntimedout = 0;
    public int conrequestwaittime = 0;
    public int averageconnwaittime = 0;
    public int totalconnrequestwaittime = 0;
    public int numconnsuccessfullymatched = 0;
    public int numconnnotsuccessfullymatched = 0;
    public int numconnpotentialconnleak = 0;
    public int waitqueuelength = 0;

    public void reset(boolean pooling, int minPoolSize) {
        this.numconnused.put(Constants.CURRENT, 0);
        this.numconnused.put(Constants.HIGHWATERMARK, 0);
        this.numconnused.put(Constants.LOWWATERMARK, 0);
        this.numconnfree.put(Constants.CURRENT, pooling ? minPoolSize : 0);
        this.numconnfree.put(Constants.HIGHWATERMARK, pooling ? minPoolSize : 1);
        this.numconnfree.put(Constants.LOWWATERMARK, 0);
        this.numconncreated = pooling ? minPoolSize : 0;
        this.numconndestroyed = 0;
        this.numconnacqired = 0;
        this.numconnreleased = 0;
        this.numconnfailedvalidation = 0;
        this.numconntimedout = 0;
        this.conrequestwaittime = 0;
        this.averageconnwaittime = 0;
        this.totalconnrequestwaittime = 0;
        this.numconnsuccessfullymatched = 0;
        this.numconnnotsuccessfullymatched = 0;
        this.numconnpotentialconnleak = 0;
        this.waitqueuelength = 0;
    }

    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("numconnused = ");
        sb.append("curr:").append(this.numconnused.get(Constants.CURRENT));
        sb.append(", high:").append(this.numconnused.get(Constants.HIGHWATERMARK));
        sb.append(", low:").append(this.numconnused.get(Constants.LOWWATERMARK)).append("\n");
        sb.append("numconnfree = ");
        sb.append("curr:").append(this.numconnfree.get(Constants.CURRENT));
        sb.append(", high:").append(this.numconnfree.get(Constants.HIGHWATERMARK));
        sb.append(", low:").append(this.numconnfree.get(Constants.LOWWATERMARK)).append("\n");
        sb.append("numconncreated = ").append(this.numconncreated).append("\n");
        sb.append("numconndestroyed = ").append(this.numconndestroyed).append("\n");
        sb.append("numconnacqired = ").append(this.numconnacqired).append("\n");
        sb.append("numconnreleased = ").append(this.numconnreleased).append("\n");
        sb.append("numconnpotentialconnleak = ").append(this.numconnpotentialconnleak).append("\n");
        return sb.toString();
    }

    public boolean compare(JDBCMonitorData data) {
        boolean result = true;
        result &= this.numconnused.get(Constants.CURRENT) == data.numconnused.get(Constants.CURRENT);
        result &= this.numconnused.get(Constants.HIGHWATERMARK) == data.numconnused.get(Constants.HIGHWATERMARK);
        result &= this.numconnused.get(Constants.LOWWATERMARK) == data.numconnused.get(Constants.LOWWATERMARK);

        result &= this.numconnfree.get(Constants.CURRENT) == data.numconnfree.get(Constants.CURRENT);
        result &= this.numconnfree.get(Constants.HIGHWATERMARK) == data.numconnfree.get(Constants.HIGHWATERMARK);
        result &= this.numconnfree.get(Constants.LOWWATERMARK) == data.numconnfree.get(Constants.LOWWATERMARK);

        result &= this.numconncreated == data.numconncreated;
        result &= this.numconndestroyed == data.numconndestroyed;

        result &= this.numconnacqired == data.numconnacqired;
        result &= this.numconnreleased == data.numconnreleased;

        result &= this.numconnpotentialconnleak == data.numconnpotentialconnleak;
        return result;
    }

    public void increment(Map<String, Integer> rangeMap) {
        increment(rangeMap, 1);
    }

    public void decrement(Map<String, Integer> rangeMap) {
        decrement(rangeMap, 1);
    }

    public void increment(Map<String, Integer> rangeMap, int addValue) {
        int current = rangeMap.get(Constants.CURRENT);
        rangeMap.put(Constants.CURRENT, current + addValue);
        updateWatermark(rangeMap);
    }

    public void decrement(Map<String, Integer> rangeMap, int subValue) {
        int current = rangeMap.get(Constants.CURRENT);
        rangeMap.put(Constants.CURRENT, current - subValue);
        updateWatermark(rangeMap);
    }

    private void updateWatermark(Map<String, Integer> rangeMap) {
        if(rangeMap.get(Constants.CURRENT) > rangeMap.get(Constants.HIGHWATERMARK)){
            rangeMap.put(Constants.HIGHWATERMARK, rangeMap.get(Constants.CURRENT));
        }
        if(rangeMap.get(Constants.CURRENT) < rangeMap.get(Constants.LOWWATERMARK)){
            rangeMap.put(Constants.LOWWATERMARK, rangeMap.get(Constants.CURRENT));
        }
    }

    @Override
    public Object clone() {
        JDBCMonitorData newInstance = new JDBCMonitorData();
        newInstance.numconnused.put(Constants.CURRENT, this.numconnused.get(Constants.CURRENT));
        newInstance.numconnused.put(Constants.HIGHWATERMARK, this.numconnused.get(Constants.HIGHWATERMARK));
        newInstance.numconnused.put(Constants.LOWWATERMARK, this.numconnused.get(Constants.LOWWATERMARK));
        newInstance.numconnfree.put(Constants.CURRENT, this.numconnfree.get(Constants.CURRENT));
        newInstance.numconnfree.put(Constants.HIGHWATERMARK, this.numconnfree.get(Constants.HIGHWATERMARK));
        newInstance.numconnfree.put(Constants.LOWWATERMARK, this.numconnfree.get(Constants.LOWWATERMARK));
        newInstance.numconncreated = this.numconncreated;
        newInstance.numconndestroyed = this.numconndestroyed;
        newInstance.numconnacqired = this.numconnacqired;
        newInstance.numconnreleased = this.numconnreleased;
        newInstance.numconnfailedvalidation = this.numconnfailedvalidation;
        newInstance.numconntimedout = this.numconntimedout;
        newInstance.conrequestwaittime = this.conrequestwaittime;
        newInstance.averageconnwaittime = this.averageconnwaittime;
        newInstance.totalconnrequestwaittime = this.totalconnrequestwaittime;
        newInstance.numconnsuccessfullymatched = this.numconnsuccessfullymatched;
        newInstance.numconnnotsuccessfullymatched = this.numconnnotsuccessfullymatched;
        newInstance.numconnpotentialconnleak = this.numconnpotentialconnleak;
        newInstance.waitqueuelength = this.waitqueuelength;
        return newInstance;
    }

}
